/**
 * Created by xiaoqiang on 2018/7/27.
 */

function setupWebViewJavascriptBridge(callback) {
    if (window.WebViewJavascriptBridge) { return callback(window.WebViewJavascriptBridge); }
    if (window.WVJBCallbacks) { return window.WVJBCallbacks.push(callback); }
    window.WVJBCallbacks = [callback];
    var WVJBIframe = document.createElement('iframe');
    WVJBIframe.style.display = 'none';
    WVJBIframe.src = 'wvjbscheme://__BRIDGE_LOADED__';
    document.documentElement.appendChild(WVJBIframe);
    setTimeout(function() { document.documentElement.removeChild(WVJBIframe) }, 0)
}

class Bridge {
    public call(cmd:string, param:Object, callback:Object):void {
        setupWebViewJavascriptBridge((b) => {
            b.callHandler(cmd, param, callback);
        });
    }
    public listen(cmd:string, callback:Object):void {
        setupWebViewJavascriptBridge((b) => {
            b.registerHandler(cmd, callback);
        });
    }
    public writeFile(name:string, data:string):void {
        this.call('save_file', {file: name, data: data}, () => {});
    }
    public readFile(name:string) {
        var defer = Q.defer();
        this.call("load_file", name, (result) => {
            if (result == 'null') {
                defer.reject(false);
            } else {
                defer.resolve(result);
            }
        });
        return defer.promise;
    }
    public deleteFile(name:string) {
        var defer = Q.defer();
        this.call("delete_file", name, (result) => {
            defer.resolve();
        });
        return defer.promise;
    }

    public debug(msg:string, prefix = 'Logger') {
        console.log(msg, prefix);
    }
    public error(msg:string, prefix = 'Logger') {
        console.error(msg, prefix);
    }

    public showToast(text, timeout?) {
        timeout = (timeout == undefined)? 2000 : timeout;
        $('#toast').html('<span>' + text + '</span>').show().animate({top: '150px'}, 'fast', 'swing', function() {
            setTimeout(function() {
                $('#toast').animate({top: '-30px'}, 'fast', 'swing', function() {
                    $(this).hide();
                })
            }, timeout);
        });
    }

    public openURL(url:string):void {
        bridge.call('open_browser', url, () => {});
    }

    public checkUpdate(uid) {
        let deffer = Q.defer();
        let url:string = 'https://www.cutterman.cn/client/checkupdate';
        $.post(url, {app: APP_NAME, version: VERSION_CODE, uid: uid, locale: LC_LANG}, (result) => {
            if (result.errno == 0) {
                if (result.update > 0) {
                    deffer.resolve(result);
                } else {
                    deffer.reject("");
                }
            }
        });
        return deffer.promise;
    }

    public download(pid:string, name:string) {
        let deffer = Q.defer();
        let url = IMAGE_URL + '/svg/' + pid + '.svg';


        var xhr = new XMLHttpRequest();
        xhr.open("GET", url, true);
        xhr.addEventListener("load", function(){
            if (xhr.status === 200) {
                //console.log(xhr.response);
                bridge.call('save_file', {file: pid + '.svg', data: xhr.response}, () => {
                    bridge.debug('download svg success');
                    bridge.call('action', {action: 'load', pid: pid, name:name}, () => {});
                    deffer.resolve();
                });
            } else {
                deffer.reject('download svg failed');
            }
        }, false);
        xhr.send();
        return deffer.promise;
    }

}

const bridge = new Bridge();

