/**
 * Created by xiaoqiang on 2018/7/27.
 */


class Message {
    command:string;
    from:string;
    email:string;
    uid:string;
    mid:string;
    extra:string;
    binary:Uint8Array;
    constructor(cmd:string, email:string, uid:string, extra?:string, bin?:Uint8Array) {
        this.mid = String((new Date()).getTime());
        this.uid = uid;
        this.command = cmd;
        this.from = 'panel';
        this.email = email;
        this.extra = extra;
        this.binary = bin;
    }

    public toJSON() {
        let obj = {
            mid: this.mid,
            command: this.command,
            from: this.from,
            email: this.email,
            uid: this.uid
        };
        if (this.extra) {
            obj['extra'] = this.extra;
        }
        return obj;
    }

    public static fromJSON(json:JSON) {
        let cmd:string = (json['command'])? json['command'] : '';
        let from:string = (json['from'])? json['from'] : '';
        let email:string = (json['email'])? json['email'] : '';
        let uid:string = (json['uid'])? json['uid'] : '';
        let extra:string = (json['extra'])? JSON.stringify(json['extra']) : '';
        let msg = new Message(cmd, email, uid, extra);
        msg.from = from;
        msg.mid = json['mid'];
        return msg;
    }
}

Vue.component('preview', {
    template: '#preview',
    props: ['user'],
    data() {
        return {
            name: 'preview component',
            ws: null,
            extraShow: true,
            toolShow: 0,
            localIP: LOCAL_IP,
            deviceIP: '',
            refreshing: false,
            status: 0,  // 0:Connecting, 1:Success, 2:fail
            tryTimes: 0,
            trialLeft: 0,
            version: VERSION,
            devices: [],
            showUserGuide: false,
            update: 0,
            isExpired: false,
            updateInfo: {
                version: '0',
                url: ''
            },
            errorCode: 0
        }
    },
    methods: {
        toolAction(index) {
            if (this.toolShow == index) {
                this.toolShow = 0;
                return;
            }
            this.toolShow = index;
        },
        onAddIp() {
            console.log('on add ip: ' + this.deviceIP);
            if (this.deviceIP.match(/^((25[0-5]|2[0-4]\d|[01]?\d\d?)($|(?!\.$)\.)){4}$/)) {
                bridge.call('add_device_ip', {ip: this.deviceIP}, () => {});
                /*
                let msg:Message = new Message('add_device_ip', this.user.email, this.user.uid, JSON.stringify({ip: this.deviceIP}));
                this.ws.send(Utils.parseSend(msg.toJSON()));
                */
            }
        },
        refreshUsb (type) {
            if (this.refreshing) { return; }
            this.refreshing = true;
            bridge.call('refresh_usb', {type: type}, () => {});
            /*
            let msg:Message = new Message('usb_refresh', this.user.email, this.user.uid, JSON.stringify({type: type}));
            this.ws.send(Utils.parseSend(msg.toJSON()));
            */
            setTimeout(() => {
                this.refreshing = false;
            }, 3000);
        },
        openBrowser() {
            bridge.openURL("http://"+this.localIP+":42211/web.html");
        },
        actionQRCode () {
            new QRCode("qrcode", {
                text: 'http://'+ this.localIP +':53330/index?uid=' + this.user.uid + '&email=' + this.user.email + '&hostIp='+ this.localIP,
                width: 120,
                height: 120,
                colorDark : "#000000",
                colorLight : "#ffffff",
                correctLevel : QRCode.CorrectLevel.H
            });
        },
        countTrialLeft() {
            if (this.user.licence != '') {
                this.trialLeft = -1;
                return;
            }
            let start = this.user.trial_start * 1000;
            let now = (new Date()).getTime();
            if (now - start > 15*24*3600*1000) { // 试用期15天
                this.trialLeft = 0;
            } else {
                this.trialLeft = Math.round((start + 15*24*3600*1000 - now)/(24*3600*1000));
            }
        },
        onMessage (messages:Message[]) {
            messages.forEach((message:Message) => {
                console.log(`on message command[${message.command}]`);
                switch (message.command) {
                    case 'document_list':
                        break;
                    case 'image':
                        if (message.binary) {
                            let extra = JSON.parse(JSON.parse(message.extra));
                            let url = "data:image/jpg;base64,"+ Utils.arrayBufferToBase64(message.binary);
                            $('#preview-image').attr('src', url);
                            this.renderImage(extra);
                        }
                        break;
                    case 'connected_devices':
                        //console.log(`connected devices[${message.extra}]`);
                        this.devices = [];
                        let items = JSON.parse(JSON.parse(message.extra));
                        items.forEach((obj) => {
                            //let obj = JSON.parse(item);
                            if (obj.os < 2 || obj.os == 'IPHONE' || obj.os == 'ANDROID') {
                                this.devices.push(obj);
                            }
                        });
                        break;
                    case 'trial_expire':
                        this.isExpired = true;
                        break;
                    case 'account_invalid':
                        bridge.showToast(rb.UI_PREVIEW_ACCOUNT_INVALID);
                        this.logout();
                        break;
                }
            });
        },
        renderImage(extra) {
            let width = parseInt(extra['width']);
            if (width < 500) {
                width = width*3;
            } else if (width < 1000) {
                width = width*1.5;
            }
            let w = document.documentElement.clientWidth;
            console.log(`panel width[${w}], image width[${width}]`);
            let val = (width > w)? '100%' : 'auto';
            $('#preview-image').css('width', val);
        },
        buy() {
            bridge.openURL(rb.UI_USER_TEXT_6 + '?app='+APP_NAME+'&token=' + this.user.token);
        },
        logout() {
            bridge.deleteFile('user.json').then(() =>{
                return Store.clean();
            }).then(() => {
                this.$emit('onlogout');
            });
        },
        checkUpdate() {
            if (Store.getValue('check_update') != null) {
                let timestamp = Store.getTimestamp('check_update');
                let now = (new Date()).getTime();
                if (now - timestamp < 24 * 3600 * 1000) {
                    return;
                }
            }

            Store.save('check_update', 1);
            bridge.checkUpdate(this.user.uid).then((result) => {
                this.update = result['update'];
                this.updateInfo.version = result['version'];
                this.updateInfo.url = result['download_url'];
            });
        },
        download() {
            bridge.openURL(this.updateInfo.url);
        },
        downloadItunes() {
            bridge.openURL('http://www.apple.com/itunes/download/');
        },
        connect() {
            console.log(`connect to local webserver`);
            this.ws = new WebSocket('ws://127.0.0.1:42200');
            this.ws.binaryType = 'arraybuffer';
            this.ws.onopen = () => {
                console.log('connected to local webserver');
                this.status = 1;
                this.tryTimes = 0;
                let msg:Message = new Message('register', this.user.email, this.user.email, JSON.stringify({
                    deviceName: 'panel',
                    deviceType: 'web',
                    dpi: '320x577',
                    ip: '127.0.0.1',
                    mac: '127.0.0.1',
                    version: '1.0'
                }));
                this.ws.send(Utils.parseSend(msg.toJSON()));
            };
            this.ws.onmessage = (msgEvt) => {
                var data = msgEvt.data;
                this.onMessage(Utils.parseReceived(data));
            };
            this.ws.onerror = (err) => {
                console.log('connect to local host error, now try');
                this.tryTimes++;
                if (this.tryTimes > 6) {
                    this.connectFail();
                    return;
                }
                setTimeout(this.connect, 2000);
            }
        },
        connectFail() {
            this.status = 2;
        },
        help() {
            bridge.openURL(rb.UI_PREVIEW_HELP_LINK + '?errorCode=' + this.errorCode);
        },
    },
    created() {
        this.localIP = LOCAL_IP;
        this.deviceIP = this.localIP.replace(/\.\d+$/, '.');

        this.connect();
    },
    mounted() {
        let h = document.documentElement.clientHeight;
        console.log('preview component created, height: ' + h);
        h = h + 20;
        $('#wrapper').css('height', h + 'px');
        $('.preview-inner').css('height', h + 'px').css('lineHeight', h + 'px');
        this.actionQRCode();
        this.countTrialLeft();

        if (Store.getValue('show_panel_guide') == null) {
            this.showUserGuide = true;
            Store.save('show_panel_guide', 1);
        }

        this.checkUpdate();
        window.initResourceBundle();
    }

});
